from stqe.host.persistent_vars import clean_var, read_var, read_env
from stqe.host.atomic_run import parse_ret, atomic_run
from libsan.host.stratis import Stratis


def generate_key():
    errors = []
    id = ""
    stratis = Stratis()

    try:
        id = "_" + str(read_env("fmf_key_id"))
    except KeyError:
        pass

    key_desc = read_var(f"STRATIS_KEY_DESC{id}")
    keyfile_name = read_var(f"STRATIS_KEYFILE{id}")

    atomic_run(f"Unsetting stratis key {key_desc}.",
               command=stratis.key_unset,
               key_desc=key_desc,
               errors=errors)

    atomic_run(f"Cleaning var STRATIS_KEYFILE{id}",
               command=clean_var,
               var=f'STRATIS_KEYFILE{id}',
               errors=errors)

    atomic_run(f"Cleaning var STRATIS_KEYFILE_PATH{id}",
               command=clean_var,
               var=f'STRATIS_KEYFILE_PATH{id}',
               errors=errors)

    atomic_run(f"Cleaning var STRATIS_KEYFILE{id}",
               command=clean_var,
               var=f'{keyfile_name}',
               errors=errors)

    atomic_run(f"Cleaning var STRATIS_KEY_DESC{id}",
               command=clean_var,
               var=f'STRATIS_KEY_DESC{id}',
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = generate_key()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thinp Metadata Spare")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        loop_dev["dev%s" % dev_num] = loopdev.create_loopdev(size=128)
        if not loop_dev["dev%s" % dev_num]:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False

    pvs = "%s %s %s %s" % (loop_dev["dev1"], loop_dev["dev2"], loop_dev["dev3"], loop_dev["dev4"])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    # Create pool without spare
    TestObj.tok("lvcreate -l10 --thin %s/pool0 --poolmetadataspare n" % vg_name)
    if not lvm.lv_info("lvol0_pmspare", vg_name):
        TestObj.tpass("lvol0_pmspare does not exist")
    else:
        TestObj.tfail("lvol0_pmspare does exist")

    # will create lvol0_pmspare with size of 4m
    TestObj.tok("lvcreate -l10 --thin %s/pool1 --poolmetadatasize 4m" % vg_name)
    lv_info = lvm.lv_info("[lvol0_pmspare]", vg_name)
    if lv_info and lv_info["size"] == "4.00m":
        TestObj.tpass("%s/lvol0_pmspare lv_size == 4.00m" % vg_name)
    else:
        if not lv_info:
            TestObj.tfail("%s/lvol0_pmspare does not exist" % vg_name)
        else:
            TestObj.tfail("%s/pool1 lv_size == %s, but expected 4.00m" % (vg_name, lv_info["size"]))

    # will change the lvol0_pmspare size to 8m
    TestObj.tok("lvcreate -l10 --thin %s/pool2 --poolmetadatasize 8m" % vg_name)
    lv_info = lvm.lv_info("[lvol0_pmspare]", vg_name)
    if lv_info and lv_info["size"] == "8.00m":
        TestObj.tpass("%s/lvol0_pmspare lvsize == 8.00m" % vg_name)
    else:
        if not lv_info:
            TestObj.tfail("%s/lvol0_pmspare does not exist" % vg_name)
        else:
            TestObj.tfail("%s/pool1 lv_size == %s, but expected 8.00m" % (vg_name, lv_info["size"]))
    TestObj.tok("lvremove -ff %s" % vg_name)

    linux.sleep(5)

    TestObj.tok("lvcreate -l10 --thin %s/pool1 --poolmetadataspare n" % vg_name)
    if not lvm.lv_info("[lvol0_pmspare]", vg_name):
        TestObj.tpass("lvol0_pmspare does not exist")
    else:
        TestObj.tfail("lvol0_pmspare does exist")

    TestObj.tok("lvcreate -l10 --thin %s/pool2 --poolmetadataspare y" % vg_name)
    lv_info = lvm.lv_info("[lvol0_pmspare]", vg_name)
    if lv_info and lv_info["size"] == "4.00m":
        TestObj.tpass("%s/lvol0_pmspare lv_size == 4.00m" % vg_name)
    else:
        if not lv_info:
            TestObj.tfail("%s/lvol0_pmspare does not exist" % vg_name)
        else:
            TestObj.tfail("%s/lvol0_pmspare lv_size == %s, but expected 4.00m" % (vg_name, lv_info["size"]))

    return True


def _clean_up():
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    start_test()

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var
from libsan.host.qemu_img import qemu_create, get_qcow_supported_options
from os import environ


def create_qemu_img():
    errors = []

    name = environ["fmf_filename"]
    size = environ["fmf_img_size"]
    supported_options = get_qcow_supported_options()
    args = dict(fmt=None, img_path="/var/tmp")
    for arg in args.keys():
        try:
            args[arg] = environ["fmf_" + arg]
        except KeyError:
            pass
    for option in supported_options:
        try:
            args[option] = environ["fmf_" + option]
        except KeyError:
            pass

    atomic_run("Creating qemu disk image %s" % name,
               filename=name,
               size=size,
               command=qemu_create,
               errors=errors,
               **args)

    atomic_run("Writing var IMAGE_NAME",
               command=write_var,
               var={"IMAGE_NAME": name},
               errors=errors)

    atomic_run("Writing var IMAGE_PATH",
               command=write_var,
               var={"IMAGE_PATH": args["img_path"]},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_qemu_img()
    exit(parse_ret(errs))



#!/usr/bin/python

# This is a reproducer for Bug 1456019
# [Pegas1.0 EA2] cache_dump crashes with SIGSEGV when passed small file as an argument

# !/usr/bin/python
from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import stqe.host.tc
from stqe.host.atomic_run import atomic_run
from libsan.host.cmdline import run
from libsan.host.linux import install_package


def cache_dump_test(TC):
    print("\n#######################################\n")
    print("INFO: Testing cache_dump for SIGSEV.")

    errors = []

    arguments = [
        {'message': "Creating very small file to dump to.", 'cmd': "fallocate -l 200 /var/tmp/small_file", 'command': run},
        {'message': "Dumping metadata from small file", 'cmd': "cache_dump /var/tmp/small_file", 'command': run},
        {'message': "Removing small file", 'cmd': 'rm -f /var/tmp/small_file', 'command': run},

    ]

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing cache_dump for SIGSEV passed.")
    else:
        TC.tfail("Testing cache_dump for SIGSEV failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def thin_dump_test(TC):
    print("\n#######################################\n")
    print("INFO: Testing thin_dump for SIGSEV.")

    errors = []

    arguments = [
        {'message': "Creating very small file to dump to.", 'cmd': "fallocate -l 200 /var/tmp/small_file", 'command': run},
        {'message': "Dumping metadata from small file", 'cmd': "thin_dump /var/tmp/small_file", 'command': run},
        {'message': "Removing small file", 'cmd': 'rm -f /var/tmp/small_file', 'command': run},
    ]

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing thin_dump for SIGSEV passed.")
    else:
        TC.tfail("Testing thin_dump for SIGSEV failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    install_package("device-mapper-persistent-data")

    cache_dump_test(TC=TC)
    thin_dump_test(TC=TC)

    if not TC.tend():
        print("FAIL: Test failed")
        sys.exit(1)

    print("PASS: Test passed")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, read_env, get_func_from_string
from stqe.host.persistent_vars import write_var, read_var
from libsan.host.linux import run
from libsan.host.lvm import lv_create, lv_deactivate, lv_convert
from libsan.host.dmpd import DMPD
from stqe.host.lvm import get_dmpd_args


def mount_lv(vg_name, lv_name):
    """
    Creates dir, filesystem and mounts LV. The purpose of this is to increase metadata volume.
    :param vg_name: VG with LV in it
    :type vg_name: string
    :param lv_name: LV name to be mounted
    :type lv_name: string
    :return: retcode
    :rtype: int
    """
    ret = run("mkdir -p /mnt/%s" % lv_name, verbose=True)
    ret += run("mkfs.ext4 /dev/%s/%s" % (vg_name, lv_name), verbose=True)
    ret += run("mount /dev/%s/%s /mnt/%s/" % (vg_name, lv_name, lv_name), verbose=True)
    return ret


def umount_lv(lv_name):
    """
    Umounts LV and removes dir.
    :param lv_name: mounted LV name
    :type lv_name: string
    :return: retcode
    :rtype: int
    """
    ret = run("umount -f /mnt/%s/" % lv_name, verbose=True)
    ret += run("rm -rf /mnt/%s" % lv_name, verbose=True)
    return ret


def metadata_snapshot(vg_name, thinpool):
    """
    Creates metadata snapshot of thinpool
    :param vg_name: VG with thinpool in it
    :type vg_name: string
    :param thinpool: thinpool name
    :type thinpool: string
    :return: retcode
    :rtype: int
    """
    ret = run("dmsetup suspend /dev/mapper/%s-%s-tpool" % (vg_name, thinpool), verbose=True)
    ret += run("dmsetup message /dev/mapper/%s-%s-tpool 0 reserve_metadata_snap" % (vg_name, thinpool), verbose=True)
    ret += run("dmsetup resume /dev/mapper/%s-%s-tpool" % (vg_name, thinpool), verbose=True)
    return ret


def create_lvs(vg_name, thinpool, lv_name, options, number_of_vols, return_output=True):
    """
    Creates few LVs to fill in pool metadata. This increases transaction ID and allows to do thin_delta
    :param vg_name: VG with thinpool in it
    :type vg_name: string
    :param thinpool: thinpool name
    :type thinpool: string
    :param lv_name: base of LVs name
    :type lv_name: string
    :param options: LV options to be used for creating LVs
    :type options: list
    :param number_of_vols: Number of LVs to create
    :type number_of_vols: int
    :param return_output: should just errors or also retcode? (default True - both)
    :type return_output: bool
    :return: retcode, list of errors
    :rtype: tuple
    """
    errors = []
    for i in range(number_of_vols):
        atomic_run("Creating thin LV No. %s" % i,
                   vg_name=vg_name + "/" + thinpool,
                   lv_name=lv_name + str(i),
                   options=options,
                   command=lv_create,
                   errors=errors)

        atomic_run("Mounting LV No. %s" % i,
                   expected_ret=0,
                   vg_name=vg_name,
                   lv_name=lv_name + str(i),
                   command=mount_lv,
                   errors=errors)

        atomic_run("Unmounting LV No. %s" % i,
                   expected_ret=0,
                   lv_name=lv_name + str(i),
                   command=umount_lv,
                   errors=errors)

        atomic_run("Deactivating thin LV No. %s" % i,
                   vg_name=vg_name,
                   lv_name=lv_name + str(i),
                   command=lv_deactivate,
                   errors=errors)
    if return_output:
        if errors == list():
            return 0, errors
        return 1, errors
    else:
        return errors


def swap_metadata(vg_name, thinpool, lv_swap, return_output=True):
    """
    Swaps thinpool metadata to another LV
    :param vg_name: VG with both thinpool and swap LV in it
    :type vg_name: string
    :param thinpool: thinpool to swap metadata from
    :type thinpool: string
    :param lv_swap: LV to swap metadata to
    :type lv_swap: string
    :param return_output: should just errors or also retcode? (default True - both)
    :type return_output: bool
    :return: retcode, list of errors
    :rtype: tuple
    """
    errors = list()
    atomic_run("Deactivating pool",
               vg_name=vg_name,
               lv_name=thinpool,
               command=lv_deactivate,
               errors=errors)

    atomic_run("Deactivating swap",
               vg_name=vg_name,
               lv_name=lv_swap,
               command=lv_deactivate,
               errors=errors)

    atomic_run("Swapping metadata",
               vg_name=vg_name,
               lv_name=lv_swap,
               options=["-y", "--thinpool %s/%s --poolmetadata" % (vg_name, thinpool)],
               command=lv_convert,
               errors=errors)
    if return_output:
        if errors == list():
            return 0, errors
        return 1, errors
    else:
        return errors


def write_data(args, errors):
    """
    Function for writing persistent var files
    :param args: any arguments supplied to general function
    :type args: dict
    :param errors: list of errors
    :type errors: list
    :return: errors
    :rtype: list
    """
    name = read_env("fmf_name")

    if "create_loopdev" in name:
        atomic_run("Writing var %s" % args['name'].upper(),
                   command=write_var,
                   var={"%s" % args['name'].upper(): "/dev/%s" % args['name']},
                   errors=errors)
        atomic_run("Writing var %s_SIZE" % args['name'].upper(),
                   command=write_var,
                   var={"%s_SIZE" % args['name'].upper(): args['size']},
                   errors=errors)

    if "create_vg" in name:
        atomic_run("Writing var VG_NAME",
                   command=write_var,
                   var={'VG_NAME': args['vg_name']},
                   errors=errors)

    if "create_lv" in name:
        atomic_run("Writing var %s" % args['lv_name'].upper(),
                   command=write_var,
                   var={args['lv_name'].upper(): args['lv_name']},
                   errors=errors)

    if "backup_metadata" in name:
        atomic_run("Writing var METADATA_BACKUP",
                   command=write_var,
                   var={'METADATA_BACKUP': args['output']},
                   errors=errors)
    return errors


def create_loopdev(**kwargs):
    """
    :param kwargs: all args taken by libsan.host.loopdev.create_loopdev
    :type kwargs: dict
    :return: return of libsan.host.loopdev.create_loopdev / None if fail
    :rtype: str / None
    """
    from libsan.host.loopdev import create_loopdev
    from libsan.host.linux import get_free_space
    from os import path

    possible_image_paths = read_env("fmf_possible_image_paths")
    image_path = None

    for possible_path in possible_image_paths:
        # check if the destination exists
        if not path.exists(possible_path):
            print("INFO: path '%s' does not exists, skipping." % possible_path)
            continue
        # comparing MB
        if (float(get_free_space(possible_path)) / 1024 ** 2) > kwargs['size']:
            image_path = possible_path
            break

    return create_loopdev(image_path=image_path, **kwargs) if image_path else None


def setup():
    """
    Basic function to merge it all together
    :return: errors
    :rtype: list
    """
    errors = []
    class_instance = DMPD()
    args = get_dmpd_args(dmpd_object=class_instance)
    args.update(get_env_args(
            {"loop_name": str, "size": int, "pv_name": str, "vg_name": str, "lv_name": str, "lv_swap": str, "path": str,
             "options": list, "number_of_vols": int, "thinpool": str, "source_vg": str, "source_lv": str,
             "source_file": str, "cmd": str, "new_name": str},
            read_var("FILE_NAMES"),
            {"loop_name": "name"}))

    command = args.pop('command')

    args['command'] = get_func_from_string(class_instance, command, local_functions=globals())
    atomic_run(errors=errors,
               **args)

    write_data(args, errors)

    return errors


if __name__ == "__main__":
    errs = setup()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.vdo import VDO
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var, write_var
from stqe.host.vdo import get_vdo_args, minimum_slab_size, maximum_logical_size, restart_modify, \
    report_modify_difference, get_underlying_device
from stqe.host.fmf_tools import get_env_args, get_func_from_string


def requires_restart_after_modification():
    return list(modification_args())


def modification_args(command=False):
    if command:
        return {"compression": "Compression",
                "deduplication": "Deduplication",
                "activate": "Activate",
                "deactivate": "Activate",
                "change_write_policy": "Configured write policy",
                "start": "Device mapper status",
                "stop": "Device mapper status",
                "grow": "size"}
    return {"ack_threads": "Acknowledgement threads",
            "bio_rotation_interval": "Bio rotation interval",
            "bio_threads": "Bio submission threads",
            "block_map_cache_size": "Block map cache size",
            "block_map_period": "Block map period",
            "cpu_threads": "CPU-work threads",
            "hash_zone_threads": "Hash zone threads",
            "logical_threads": "Logical threads",
            "max_discard_size": "Max discard size",
            "physical_threads": "Physical threads"}


def commands_changing_stats():
    return ["compression", "deduplication", "activate", "deactivate", "change_write_policy", "stop", "start", "grow"]


def list_not_changing(**kwargs):
    from libsan.host.lvm import lv_create, lv_remove

    def _create_lvs(vg, count, errors):
        for i in range(count):
            atomic_run("Creating LV lv%s" % i,
                       command=lv_create,
                       lv_name="lv%s" % i,
                       vg_name=vg,
                       options=["-l %s%%VG" % int(100 / count)],
                       errors=errors)
        print("INFO: All LVs created.")

    def _remove_lvs(vg, count, errors):
        for i in range(count):
            atomic_run("Removing LV lv%s" % i,
                       command=lv_remove,
                       lv_name="lv%s" % i,
                       vg_name=vg,
                       errors=errors)
        print("INFO: All LVs removed.")

    def _create_vdos(vdo, vg, count, errors):
        for i in range(count):
            atomic_run("Creating VDO vdo%s" % i,
                       command=vdo.create,
                       name="vdo_list%s" % i,
                       device="/dev/mapper/%s-lv%s" % (vg, i),
                       slab_size=minimum_slab_size("/dev/mapper/%s-lv%s" % (vg, i), default_to_2g=False),
                       errors=errors)
        print("INFO: All VDOs created.")

    def _remove_vdos(vdo, count, errors):
        for i in range(count):
            atomic_run("Removing VDO vdo%s" % i,
                       command=vdo.remove,
                       force=True,
                       name="vdo_list%s" % i, errors=errors)
        print("INFO: All VDOs removed.")

    errors = []

    vg = read_var("VG_NAME")
    _create_lvs(vg, kwargs["count"], errors)

    vdo = VDO(disable_check=True)
    _create_vdos(vdo, vg, kwargs["count"], errors)
    x = ""
    ret = 0
    vdo = VDO(disable_check=True)
    for i in range(kwargs["iterations"]):
        print("I: %s" % i)
        _, y = vdo.list(return_output=True)
        if i == 0:
            x = y
            continue
        if x != y:
            ret = 1
            e = "##### DIFFERENT LISTINGS, iteration %s #####\n" % i
            e += "X: %s\n" % x
            e += "#####\n"
            e += "Y: %s\n" % y
            errors.append(e)
            break
    out = "Checking listing passed %s iterations." % kwargs["iterations"] if errors == [] else "/n".join(errors)
    _remove_vdos(vdo, kwargs["count"], errors)
    _remove_lvs(vg, kwargs["count"], errors)
    print(out, type("out"))
    return ret, out


def grow_physical(return_output=True):
    from libsan.host.lvm import lv_extend
    errors = []
    lv = read_var("LV_THIN")
    vg = read_var("VG_NAME")
    vdo = VDO(disable_check=True)
    min_slab_size = minimum_slab_size("/dev/mapper/%s-%s" % (vg, lv), default_to_2g=False)
    arguments = [
        {'message': "Creating VDO to be grown", 'command': vdo.create, 'name': "vdo_grow",
         'device': "/dev/mapper/%s-%s" % (vg, lv), 'force': True, 'slab_size': min_slab_size},
        {'message': "Growing LV under VDO to have extra physical size", 'command': lv_extend, 'vg_name': vg,
         'lv_name': lv, 'options': ['-L +%s' % (str(int(min_slab_size[:-1]) * 3) + "M")]},
        {'message': "Growing physical size", 'command': vdo.grow, 'grow_type': "physical", 'name': "vdo_grow"},
        {'message': "Removing VDO device", 'command': vdo.remove, 'name': "vdo_grow"}
        # TODO: Reduce the LV to original size
    ]
    ret = 0
    for argument in arguments:
        ret_cycle = atomic_run(errors=errors,
                               **argument)
        if ret_cycle != 0:
            ret = 1
    return (ret, "/n".join(errors)) if return_output else ret


def call_func(args, vdo_object, errors):
    # slab size checking
    if "create" in read_env("fmf_name") and "success" in read_env("fmf_name") and "slab_size" not in args:
        args["slab_size"] = minimum_slab_size(args['device'])
        if args["slab_size"] == "2G":
            # 2G is default, no need to specify it
            args.pop('slab_size')

    if "slab_size" in args and args['slab_size'] == "minimum":
        args["slab_size"] = minimum_slab_size(args['device'], default_to_2g=False)

    if "logical_size" in args and args['logical_size'] == "maximum":
        args['logical_size'] = maximum_logical_size()

    if ("vdo_cli/modify" in read_env("fmf_name") or "vdo_cli/various_commands" in read_env("fmf_name")) and \
            "success" in read_env("fmf_name"):
        extra_args = {"name": args['name']} if "name" in args else {}
        extra_args = {"conf_file": args['conf_file']} if "conf_file" in args else extra_args
        _, args["status_before"] = atomic_run("Getting VDO status after modifications",
                                              return_output=True,
                                              verbosity=False,
                                              command=vdo_object.status,
                                              errors=errors,
                                              **extra_args)

    if "various_commands/remove/success" in read_env("fmf_name"):
        device = get_underlying_device(name=args["name"],
                                       conf_file=args["conf_file"] if "conf_file" in args else "/etc/vdoconf.yml")
        if device:
            atomic_run("Replacing var VDO_DEVICE with underlying device.",
                       command=write_var,
                       var={'VDO_DEVICE': device},
                       errors=errors)
    return args


def vdo_cli():
    errors = []
    vdo = VDO(disable_check=True)
    args = get_vdo_args(vdo_object=vdo)
    # some extra args for some setups
    args.update(get_env_args({"path": str, "enable": None, "iterations": int, "count": int, "vg_name": None}))

    args = call_func(args, vdo, errors)
    if args is None:
        return errors
    status_before = args.pop("status_before") if "status_before" in args else ""

    command_string = args['command']
    args['command'] = get_func_from_string(vdo, args.pop('command'), local_functions=globals())

    ret = atomic_run(errors=errors,
                     **args)

    check_clean(ret=ret, vdo_object=vdo, args=args, errors=errors, status_before=status_before, command=command_string)

    return errors


def check_clean(**kwargs):
    try:
        if read_env("fmf_requires_restart") == True:
            restart_modify(vdo_object=kwargs["vdo_object"], vdo_name=kwargs["args"]['name'], errors=kwargs["errors"])
    except KeyError:
        pass
    if ("vdo_cli/modify" in read_env("fmf_name") or kwargs["command"] in commands_changing_stats()) and \
            "success" in read_env("fmf_name") and not \
            any([True for x in ["log_file", "conf_file", "verbose"] if x in read_env("fmf_name")]):
        name = {"name": kwargs["args"]['name']} if "name" in kwargs["args"] else {}
        _, status_after = atomic_run("Getting VDO status after change",
                                     return_output=True,
                                     verbosity=False,
                                     command=kwargs["vdo_object"].status,
                                     errors=kwargs["errors"],
                                     **name)
        changed_vars_name = []
        changed_args = []
        is_command = True if kwargs["command"] in commands_changing_stats() else False

        for arg in list(kwargs["args"]) + [kwargs["command"]]:
            if arg in modification_args(is_command):
                changed_vars_name.append(modification_args(is_command)[arg])
                changed_args.append(arg)

        for i, changed_var_name in enumerate(changed_vars_name):
            report_modify_difference(errors=kwargs["errors"], status_old=kwargs["status_before"],
                                     status_new=status_after, changed_var=changed_var_name,
                                     changed_argument=changed_args[i])
    return


if __name__ == "__main__":
    errs = vdo_cli()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.linux as linux
import libsan.host.loopdev as loopdev

from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def _pool():
    global TestObj, vg_name

    TestObj.tok("lvcreate -l20 -T %s/pool" % vg_name)
    # redirect to /dev/null to ignore "yes: standard output: Broken pipe" message
    # see: http://stackoverflow.com/questions/20573282/hudson-yes-standard-output-broken-pipe
    TestObj.tok("yes 2>/dev/null | lvremove %s/pool" % vg_name)

    TestObj.tok("lvcreate -l20 -T %s/pool" % vg_name)
    TestObj.tok("lvremove -f %s/pool" % vg_name)

    TestObj.tok("lvcreate -l20 -T %s/pool" % vg_name)
    TestObj.tok("lvremove -ff %s/pool" % vg_name)

    TestObj.tok("lvcreate -l20 -T %s/pool" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    TestObj.tok("lvcreate -l20 -V 100m -T %s/pool -n lv1" % vg_name)
    TestObj.tok("lvremove -ff %s/lv1" % vg_name)

    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv1" % vg_name)
    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv2" % vg_name)
    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv3" % vg_name)
    TestObj.tok("lvremove -ff %s/lv1 %s/lv2 %s/lv3" % (vg_name, vg_name, vg_name))
    check_lv_expected_value(TestObj, "pool", vg_name, {"data_percent": "0.00"})

    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv1" % vg_name)
    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv2" % vg_name)
    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv3" % vg_name)
    TestObj.tok("lvremove -ff /dev/%s/lv[1-3]" % vg_name)

    TestObj.tok("lvcreate -V 100m -T %s/pool -n lv1" % vg_name)
    TestObj.tok("lvcreate -s %s/lv1 -n snap1" % vg_name)
    TestObj.tok("lvcreate -s %s/snap1 -n snap2" % vg_name)
    TestObj.tok("lvremove -ff %s/snap1 %s/snap2" % (vg_name, vg_name))


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Remove test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=256)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _pool()

    return True


def _clean_up():
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from libsan.host.lio import TargetCLI
from libsan.host.cmdline import run
import os


def delete_backstore():
    errors = []

    arguments = {
        "wwn": None,
        "lun": None,
        "ip_address": None,
        "ip_port": None,
        "tag": None
    }

    fileio_file = None
    path = os.environ["fmf_backstore"]
    if path == "fileio":
        fileio_file = read_var("FILEIO_FILE")

    for arg in arguments.keys():
        try:
            arguments[arg] = os.environ['fmf_' + arg]
        except KeyError:
            pass

    arguments["name"] = read_var(path.upper() + "_NAME")

    target = TargetCLI(path="/backstores/%s" % path)

    atomic_run("Deleting bacstore %s" % arguments["name"],
               command=target.delete,
               errors=errors,
               **target.remove_nones(arguments)
               )

    if fileio_file and os.path.isfile(fileio_file):
        atomic_run("Removing file ",
                   cmd="rm -f %s" % fileio_file,
                   command=run,
                   errors=errors
                   )

    return errors


if __name__ == "__main__":
    errs = delete_backstore()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config, get_local_disk_data
from os import environ
from stqe.host.persistent_vars import read_var


def volume_ident_led_success():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, disks = atomic_run("Listing local disks before changing ident LED.",
                                command=lsm.local_disk_list,
                                return_output=True,
                                script=True,
                                errors=errors)
        data_before = get_local_disk_data(disks)

        atomic_run("Enabling ident LED of volume %s." % vol,
                   command=lsm.volume_ident_led_on,
                   vol=vol,
                   errors=errors)

        atomic_run("Disabling ident LED of volume %s." % vol,
                   command=lsm.volume_ident_led_off,
                   vol=vol,
                   errors=errors)

        ret, disks = atomic_run("Listing local disks after changing ident LED.",
                                command=lsm.local_disk_list,
                                return_output=True,
                                script=True,
                                errors=errors)
        data_after = get_local_disk_data(disks)

        if data_before != data_after:
            atomic_run("Enabling ident LED to return it to previous state with protocol %s." % config['protocol'],
                       command=lsm.volume_ident_led_on,
                       vol=vol,
                       errors=errors)
            ret, disks = atomic_run("Listing local disks after returning ident LED state.",
                                    command=lsm.local_disk_list,
                                    return_output=True,
                                    script=True,
                                    errors=errors)
            data_after = get_local_disk_data(disks)
            if data_before != data_after:
                msg = "FAIL: Could not return ident LED to previous state with protocol %s" % config['protocol']
                print(msg)
                errors.append(msg)

    return errors


def volume_ident_led_on_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail enabling ident LED for volume without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.volume_ident_led_on},
            {'message': "Trying to fail enabling ident LED for volume with WRONG volume with protocol %s" %
                        config['protocol'], 'vol': "WRONG", 'command': lsm.volume_ident_led_on}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_ident_led_off_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail disabling ident LED for volume without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.volume_ident_led_off},
            {'message': "Trying to fail disabling ident LED for volume with WRONG volume with protocol %s" %
                        config['protocol'], 'vol': "WRONG", 'command': lsm.volume_ident_led_off}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_ident_led_offline():
    errors = []
    # TODO: Check changing ident LED when volume is disabled, create test
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_ident_led_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_ident_led_on_fail()
        errs += volume_ident_led_off_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.persistent_vars import read_var, write_var
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI
from random import randint
import string
import secrets
from os import environ


def remove_nones(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


def rand_string(length):
    """
    Creates random string of length passed as parameter.
    :param length: integer representing length of the string
    :return:
    None: if length is not integer
    string: with random characters
    """

    if not isinstance(length, int):
        return None

    alphabet = string.ascii_letters + string.digits + '.-+@_=:/[],~'  # rfc7143#section-6.1
    # Allowed in targetcli:
    # https://github.com/open-iscsi/configshell-fb/blob/b4923ee5591d8b980003150e0ba6ffe512d8c9da/configshell/shell.py#L116
    generated_string = ''.join(secrets.choice(alphabet) for _ in range(length))

    return str(generated_string)


def configure_chap():
    errors = []
    target_iqn = read_var("TARGET_IQN")
    tpg = read_var("TPG")
    # supported fmf arguments
    args = {"random": False, "userid": None, "password": None,
            "mutual_userid": None, "mutual_password": None, "chap_ways": 1}
    # Getting arguments from fmf
    for arg in args.keys():
        try:
            if arg == "chap_ways":
                args[arg] = int(environ["fmf_%s" % arg])
                continue
            args[arg] = environ["fmf_%s" % arg]
        except KeyError:
            pass

    if args["random"]:  # if random is true, generate userid and password randomly
        if args["chap_ways"] == 2:
            args["mutual_userid"] = rand_string(randint(1, 255))
            args["mutual_password"] = rand_string(randint(1, 255))
        args["userid"] = rand_string(randint(1, 255))
        args["password"] = rand_string(randint(1, 255))
    for i in ["random", "chap_ways"]:  # removing these args from dict because this dict is used as kwargs in atomic run
        args.__delitem__(i)

    target = TargetCLI()

    target.path = "/iscsi"

    atomic_run("Enabling discovery_auth and setting userid and password",
               enable=1,
               group="discovery_auth",
               command=target.set,
               errors=errors,
               **remove_nones(args)
               )

    target.path = "/iscsi/" + target_iqn + "/" + tpg

    atomic_run("Enabling authentication and generate node acls",
               group="attribute",
               authentication=1,
               generate_node_acls=1,
               command=target.set,
               errors=errors
               )

    atomic_run("Setting userid and password in auth group",
               group="auth",
               command=target.set,
               errors=errors,
               **remove_nones(args)
               )

    if args["mutual_userid"] is not None and args["mutual_password"] is not None:
        atomic_run("Writing var CHAP_MUTUAL_USERID",
                   command=write_var,
                   var={"CHAP_MUTUAL_USERID": args["mutual_userid"]},
                   errors=errors)

        atomic_run("Writing var CHAP_MUTUAL_PASSWORD",
                   command=write_var,
                   var={"CHAP_MUTUAL_PASSWORD": args["mutual_password"]},
                   errors=errors)

    atomic_run("Writing var CHAP_USERID",
               command=write_var,
               var={"CHAP_USERID": args["userid"]},
               errors=errors)

    atomic_run("Writing var CHAP_PASSWORD",
               command=write_var,
               var={"CHAP_PASSWORD": args["password"]},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = configure_chap()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var
from libsan.host.lvm import vg_create, lv_create, vg_remove, pv_remove
from libsan.host.linux import mkdir, rmdir, mkfs, mount, umount, get_system_logs
import time

start = time.mktime(time.localtime())

error = list()

device = read_var("LOOP1")
if not device:
    error.append("FAIL: Could not get the device from read_var")
    exit(parse_ret(error))
atomic_run("the device is loop1",
           command=run,
           cmd="lsblk | grep %s" % device.split('/')[2],
           return_output=True,
           errors=error)

# SETUP
parted = ["parted -s %s mklabel MSDOS" % device,
          "parted -s %s mkpart pri 1 1248" % device,
          "parted -s %s mkpart ext 1249 3814" % device,
          "parted -s %s mkpart log 1249 1728" % device,
          "parted -s %s mkpart log 1729 1760" % device,
          "parted -s %s mkpart log 1761 2016" % device,
          "parted -s %s mkpart log 2017 2047" % device,
          "parted -s %s mkpart log 2048 2687" % device,
          "parted -s %s mkpart log 2688 3007" % device,
          "parted -s %s mkpart log 3008 3320" % device,
          "parted -s %s mkpart log 3321 3336" % device,
          "parted -s %s mkpart log 3337 3814" % device]
for cmd in parted:
    atomic_run("Running parted cmd '%s'" % cmd,
               command=run,
               cmd=cmd,
               errors=error)
atomic_run("Running udevadm trigger",
            command=run,
            cmd="udevadm trigger",
            errors=error)
atomic_run("Running lsblk cmd",
            command=run,
            cmd="lsblk | grep loop",
            errors=error)
atomic_run("Creating VG 'vgtest'",
           command=vg_create,
           vg_name="vgtest",
           pv_name="%s" % device + (" %s" % device).join(["p" + str(x) for x in [1, 5, 8, 9, 10, 11, 13]]),
           errors=error)
atomic_run("Creating LV",
           command=lv_create,
           lv_name="testvol",
           vg_name="vgtest",
           options=["-l 100%FREE", "-y"],
           errors=error)
atomic_run("Creating filesystem",
           command=mkfs,
           device_name="/dev/mapper/vgtest-testvol",
           fs_type="xfs",
           errors=error)
atomic_run("Creating mount dir",
           command=mkdir,
           new_dir="/tmp/test_mount",
           errors=error)
atomic_run("Mounting to /tmp/test_mount",
           command=mount,
           device="/dev/mapper/vgtest-testvol",
           mountpoint="/tmp/test_mount",
           errors=error)
# TEST

atomic_run("Creating test file",
           command=run,
           cmd="dd if=/dev/urandom of=/tmp/test_mount/test_file bs=3K count=1128966 status=progress",
           expected_ret=1,
           expected_out="No space left on device",
           errors=error)
atomic_run("Syncing mount point",
           command=run,
           cmd="sync /tmp/test_mount",
           errors=error)
atomic_run("Removing test file",
           command=run,
           cmd="rm -f /tmp/test_mount/test_file",
           errors=error)
atomic_run("Syncing mount point",
           command=run,
           cmd="sync /tmp/test_mount",
           errors=error)
atomic_run("Sending trim",
           command=run,
           cmd="fstrim /tmp/test_mount",
           expected_out=read_env("fmf_expected_fstrim_out"),
           expected_ret=read_env("fmf_expected_fstrim_ret"),
           errors=error)

ret, syslogs = atomic_run(message="Checking logs for errors.",
                          command=get_system_logs,
                          since="-%s" % int(time.mktime(time.localtime()) - start),
                          return_output=True,
                          errors=error)
sys_error = "attempt to access beyond end of device"
if sys_error in syslogs:
    error.append("FAIL: Found error '%s' in syslog." % sys_error)

# CLEANUP
atomic_run("Umounting/tmp/test_mount",
           command=umount,
           mountpoint="/tmp/test_mount",
           errors=error)
atomic_run("Removing mount dir",
           command=rmdir,
           dir_name="/tmp/test_mount",
           errors=error)
atomic_run("Removing VG 'vgtest'",
           command=vg_remove,
           vg_name="vgtest",
           force=True,
           errors=error)
for pv in [" %sp" % device + str(x) for x in [1, 5, 8, 9, 10, 11, 13]]:
    atomic_run("Removing PV %s" % pv,
               command=pv_remove,
               pv_name=pv,
               errors=error)
atomic_run("Removing partitioning table",
           command=run,
           cmd="dd if=/dev/zero of=%s bs=512 count=1" % device,
           errors=error)
atomic_run("Reloading partitioning table",
           command=run,
           cmd="partprobe %s" % device,
           errors=error)

exit(parse_ret(error))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_replicate_range_success():
    errors = []

    src_vol = read_var("LSM_VOL_ID")
    dst_vol = read_var("LSM_VOL_ID_2")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        rep_types = ["CLONE", "COPY"]
        if 'ontap' in config['protocol']:
            rep_types = ["CLONE"]  # copy is not supported on ontap yet

        for rep_type in rep_types:
            atomic_run("Replicating volume %s to volume %s with type %s with protocol %s." %
                       (src_vol, dst_vol, rep_type, config['protocol']),
                       command=lsm.volume_replicate_range,
                       src_vol=src_vol,
                       dst_vol=dst_vol,
                       rep_type=rep_type,
                       src_start=0,
                       dst_start=0,
                       count=100,
                       force=True,
                       errors=errors)

            atomic_run("Removing dependency between replicated volumes %s and %s" % (src_vol, dst_vol),
                       command=lsm.volume_dependants_rm,
                       vol=src_vol,
                       errors=errors)

            atomic_run("Replicating volume %s to volume %s with type %s with repeated arguments with protocol %s." %
                       (src_vol, dst_vol, rep_type, config['protocol']),
                       command=lsm.volume_replicate_range,
                       src_vol=src_vol,
                       dst_vol=dst_vol,
                       rep_type=rep_type,
                       src_start=[0, 100, -100],
                       dst_start=[0, 200, -100],
                       count=[100, 1, -100],
                       force=True,
                       errors=errors)

            atomic_run("Removing dependency between replicated volumes %s and %s" % (src_vol, dst_vol),
                       command=lsm.volume_dependants_rm,
                       vol=src_vol,
                       errors=errors)
    return errors


def volume_replicate_range_fail():
    errors = []

    src_vol = read_var("LSM_VOL_ID")
    dst_vol = read_var("LSM_VOL_ID_2")
    rep_type = "CLONE"

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail replicating volume range without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range without source volume with protocol %s" %
                        config['protocol'], 'dst_vol': dst_vol, 'src_start': 0, 'dst_start': 0, 'count': 100,
             'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range without destination volume with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'src_start': 0, 'dst_start': 0, 'count': 100,
             'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range without replication type volume with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'src_start': 0, 'dst_start': 0, 'count': 100,
             'dst_vol': dst_vol, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range without source start with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'dst_vol': dst_vol, 'dst_start': 0, 'count': 100,
             'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range without destination start with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'src_start': 0, 'dst_vol': dst_vol, 'count': 100,
             'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range without block count with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'src_start': 0, 'dst_start': 0, 'dst_vol': dst_vol,
             'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range with wrong source volume with protocol %s" %
                        config['protocol'], 'src_vol': "WRONG", 'dst_vol': dst_vol, 'src_start': 0, 'dst_start': 0,
             'count': 100, 'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range with wrong destination volume with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'dst_vol': "WRONG", 'src_start': 0, 'dst_start': 0,
             'count': 100, 'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range with wrong source start with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'dst_vol': dst_vol, 'src_start': "WRONG",
             'dst_start': 0,
             'count': 100, 'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range with wrong destination start with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'dst_vol': dst_vol, 'src_start': 0,
             'dst_start': "WRONG",
             'count': 100, 'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range with wrong block count with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'dst_vol': dst_vol, 'src_start': 0, 'dst_start': 0,
             'count': "WRONG", 'rep_type': rep_type, 'command': lsm.volume_replicate_range},
            {'message': "Trying to fail replicating volume range with wrong replication type with protocol %s" %
                        config['protocol'], 'src_vol': src_vol, 'dst_vol': dst_vol, 'src_start': 0, 'dst_start': 0,
             'count': 100, 'rep_type': "WRONG", 'command': lsm.volume_replicate_range}
        ]
        for argument in arguments:
            ret, data = atomic_run(expected_ret=2,
                                   errors=errors,
                                   return_output=True,
                                   **argument)
            if ret == 0:
                atomic_run("Removing dependency between replicated volumes %s and %s after not failing" %
                           (src_vol, dst_vol),
                           command=lsm.volume_dependants_rm,
                           vol=src_vol,
                           errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_replicate_range_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_replicate_range_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI
from os import environ


def saveconfig_create():
    errors = []
    target = TargetCLI(path="")

    args = dict(savefile=None)
    try:
        args["savefile"] = environ['fmf_savefile_name']
    except KeyError:
        pass

    atomic_run("Creating saveconfig",
               command=target.saveconfig,
               errors=errors,
               **target.remove_nones(args)
               )

    return errors


if __name__ == "__main__":
    errs = saveconfig_create()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from time import sleep
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var
from libsan.host.fcoe import unconfigure_soft_fcoe


def cleanup_fcoe():
    print("INFO: Cleaning up soft FCoE.")
    errors = []

    print("Waiting for data stream to settle before cleaning up FCoE.")
    sleep(5)

    atomic_run("Cleaning up FCoE",
               command=unconfigure_soft_fcoe,
               errors=errors)

    atomic_run("Cleaning var VDO_DEVICE",
               command=clean_var,
               var="VDO_DEVICE",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_fcoe()
    exit(parse_ret(errs))



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')

version = {}
with open("stqe/_version.py") as fp:
    exec(fp.read(), version)

setup(
    name='stqe',
    version=version['__version__'],
    description='Python modules used by kernel storage QE team',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/rh-kernel-stqe/python-stqe',
    author='Bruno Goncalves',
    author_email='bgoncalv@redhat.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: POSIX :: Linux',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='stqe, python-stqe, stqe-test',
    packages=find_packages(
        include=['stqe*']
    ),
    python_requires='>=3.6',
    install_requires=['libsan', 'python-augeas', 'fmf==1.1.0', 'pexpect'],
    package_data={
        "": ["*.py", "*.fmf", "*.conf", "*.cfg", ".fmf/version"]
    },
    data_files=[('/etc/', ['multipath.conf', 'san_top.conf']),
                ('/usr/share/man/man1', ['man/stqe-test.1'])],
    scripts=["bin/stqe-test", "bin/stqe-tool", "bin/fc_tool"],
    project_urls={
        'Bug Reports': 'https://gitlab.com/rh-kernel-stqe/python-stqe/issues',
        'Source': 'https://gitlab.com/rh-kernel-stqe/python-stqe',
    },
)




#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def setup_crypt():
    disk = read_var("VDO_DEVICE")
    print("INFO: Setting up dm-crypt on device %s." % disk)
    errors = []

    name = "vdo_crypted"

    atomic_run(message="Creating key",
               command=run,
               cmd="echo -e  'y\\n'| ssh-keygen -f key -N redhat",
               errors=errors)

    atomic_run(message="Encrypting disk with dm-crypt/LUKS.",
               command=run,
               cmd="cryptsetup luksFormat %s key --batch-mode" % disk,
               errors=errors)

    atomic_run(message="Opening encrypted disk.",
               command=run,
               cmd="cryptsetup luksOpen %s %s -d key" % (disk, name),
               errors=errors)

    atomic_run("Writing var VDO_DEVICE",
               command=write_var,
               var={'VDO_DEVICE': "/dev/mapper/%s" % name},
               errors=errors)

    atomic_run("Writing var CRYPTED_DISK",
               command=write_var,
               var={'CRYPTED_DISK': "%s" % disk},
               errors=errors)

    atomic_run(message="Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_crypt()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_snap_restore_success():
    errors = []

    snap_id = read_var("LSM_FS_SNAP_ID")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Restoring FS %s from snapshot %s with protocol %s." % (fs_id, snap_id, config['protocol']),
                   command=lsm.fs_snap_restore,
                   fs=fs_id,
                   snap=snap_id,
                   force=True,
                   errors=errors)
        # FIXME: Add test with 'lsm_file' and 'fileas'
    return errors


def fs_snap_restore_fail():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    snap_id = read_var("LSM_FS_SNAP_ID")
    snap_name = read_var("LSM_FS_SNAP_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail restoring FS from snapshot without any paramethers with protocol %s" % config[
                'protocol'], 'command': lsm.fs_snap_restore},
            {'message': "Trying to fail restoring FS from snapshot without 'snap' with protocol %s" % config[
                'protocol'], 'command': lsm.fs_snap_restore, 'fs': fs_id},
            {'message': "Trying to fail restoring FS from snapshot without 'fs' with protocol %s" % config[
                'protocol'], 'command': lsm.fs_snap_restore, 'snap': snap_id},
            {'message': "Trying to fail restoring FS from snapshot with wrong 'snap' with protocol %s" %
                        config['protocol'], 'command': lsm.fs_snap_restore, 'fs': fs_id, 'snap': "wrong"},
            {'message': "Trying to fail restoring FS from snapshot with wrong 'fs' with protocol %s" %
                        config['protocol'], 'command': lsm.fs_snap_restore, 'fs': "wrong", 'snap': snap_id},
            {'message': "Trying to fail restoring FS from snapshot by giving 'snap' name "
                        "instead of ID with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_restore, 'snap': snap_name, 'fs': fs_id},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_snap_restore_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_snap_restore_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

import os
import libsan.host.linux as linux
import libsan.host.targetd as targetd
from time import sleep
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret


def targetd_ssl_func_test():
    errors = []

    password = os.environ["fmf_ssl_password"]

    atomic_run("Starting targetd SSL NO KEY test",
               enable=True,
               command=targetd.ssl_change,
               errors=errors
               )

    atomic_run("Adding password to the config",
               password=password,
               command=targetd.set_password,
               errors=errors
               )

    atomic_run("Trying to start targetd without SSL key, it should fail!",
               service_name="targetd",
               command=linux.service_restart,
               errors=errors
               )

    print("Waiting 10 seconds for the service to restart")
    sleep(10)

    atomic_run("Getting targetd service status",
               retcode=3,
               command=targetd.targetd_status,
               errors=errors
               )

    atomic_run("Removing password from config",
               command=targetd.set_password,
               errors=errors
               )

    cmd = "systemctl status targetd -l"
    has_systemctl = True

    if run("which systemctl", verbose=False) != 0:
        has_systemctl = False
    if not has_systemctl:
        cmd = "service targetd status -l"

    ret, data = run(cmd, return_output=True)
    for line in data.splitlines():
        if "ERROR:root:SSL file: '/etc/target/targetd_key.pem' does not exist" in line:
            return errors

    print("FAIL: Service has failed with different error than expected or service is running!")
    errors.append("FAIL: Service has failed with different error than expected or service is running!")

    return errors


if __name__ == "__main__":
    errs = targetd_ssl_func_test()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.vdo import VDO
from libsan.host.fio import fio_stress
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from os import environ
from stqe.host.vdo import is_block_device


def xfs_stress():
    errors = []
    vdo = VDO(disable_check=True)

    fs_dir = read_var("FS_DIR")
    vdo_name = environ['fmf_vdo_name']

    ret = is_block_device("/dev/mapper/%s" % vdo_name)
    if ret != True:
        errors.append(ret)
        return errors

    arguments = [
        {'message': "Checking vdo status to get info on data.", 'command': vdo.status, 'name': vdo_name},
        {'message': "Starting fio stress on FS on VDO",
         'command': fio_stress,
         'of': "%s/test.img" % fs_dir,
         'verbose': True,
         # fio settings
         'ioengine': environ['fmf_ioengine'],
         'direct': environ['fmf_direct'],
         'group_reporting': environ['fmf_group_reporting'],
         'bs': environ['fmf_bs'],
         'iodepth': environ['fmf_iodepth'],
         'numjobs': environ['fmf_numjobs'],
         'rw': environ['fmf_rw'],
         'rwmixread': environ['fmf_rwmixread'],
         'randrepeat': environ['fmf_randrepeat'],
         'norandommap': environ['fmf_norandommap'],
         'size': environ['fmf_size'],
         'runtime': int(environ['fmf_runtime'])*60},
        {'message': "Checking vdo status to get info on data.", 'command': vdo.status, 'name': vdo_name}
    ]

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = xfs_stress()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.loopdev import create_loopdev, get_loopdev
from stqe.host.persistent_vars import write_var, read_var, read_env


def setup_loop_devs(number_of_loopdevs):
    errors = []

    if not number_of_loopdevs:
        number_of_loopdevs = 2

    for i in range(number_of_loopdevs):
        atomic_run("Creating loop device number %s" % i,
                   command=create_loopdev,
                   errors=errors
                   )

    loopdevs = atomic_run(message="Gettting loop devices",
                          command=get_loopdev,
                          errors=errors)

    blockdevs = read_var("STRATIS_DEVICE")

    if blockdevs:
        # backup the previous devices
        atomic_run("Writing var STRATIS_DEVICE_BACKUP",
                   command=write_var,
                   var={'STRATIS_DEVICE_BACKUP': " ".join(blockdevs)},
                   errors=errors)
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        loopdevs += [x for x in blockdevs if x not in blockdevs]

    if number_of_loopdevs and len(loopdevs) < number_of_loopdevs:
        msg = "WARN: Found only %s disks, need %s disks." % (len(loopdevs), number_of_loopdevs)
        print(msg)
        errors.append(msg)

    print("Using these blockdevs: %s" % " ".join(loopdevs))
    for disk in loopdevs:
        atomic_run("Zeroing superblock of disk %s." % disk,
                   command=run,
                   cmd="dd if=/dev/zero of=%s bs=1M count=10" % disk,
                   errors=errors)
        atomic_run("remove multipath superblock of disk %s." % disk,
                   command=run,
                   cmd="multipath -W %s" % disk,
                   errors=errors)

    atomic_run("Writing var STRATIS_DEVICE",
               command=write_var,
               var={'STRATIS_DEVICE': loopdevs},
               errors=errors)

    atomic_run(message="Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    try:
        number_of_loopdevs = read_env("fmf_number_of_disks")
    except KeyError:
        number_of_loopdevs = None
    errs = setup_loop_devs(number_of_loopdevs)
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

#
# Bugs related:
#   BZ1404736 - hang on mkfs a 300T Thinp device 
#

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.linux as linux
import libsan.host.loopdev as loopdev
import libsan.misc.time as time
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"

lv_mnt = "/mnt/lv"


def _discard():
    global TestObj, vg_name

    TestObj.tok("lvcreate -L1G -V1T -T %s/pool -n discard" % vg_name)

    filesystem = linux.get_default_fs()

    print("INFO: Measuring time of mkfs without discard")
    no_discard_start_time = time.get_time(in_seconds=True)
    TestObj.tok("mkfs.%s -K /dev/mapper/testvg-discard" % filesystem)
    no_discard_end_time = time.get_time(in_seconds=True)

    lvm.lv_show()

    no_discard_time = no_discard_end_time - no_discard_start_time

    TestObj.trun("dd if=/dev/zero of=/dev/mapper/testvg-discard count=1k bs=1k")

    print("INFO: Measuring time of mkfs with discard")
    start_time = time.get_time(in_seconds=True)
    TestObj.tok("mkfs.%s /dev/mapper/testvg-discard" % filesystem)
    end_time = time.get_time(in_seconds=True)

    lvm.lv_show()

    # Make sure the mkfs with discard do not take too much more time than without discard
    max_time = no_discard_time + (no_discard_time * 0.2)
    if end_time < (start_time + max_time):
        TestObj.tpass("discard completed in less than %ss" % max_time)
    else:
        TestObj.tfail("discard completed in more than %ss (%ss)" % (max_time, end_time - start_time))

    TestObj.tok("lvremove -ff %s" % vg_name)


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Discard Perf test")
    print(80 * "#")

    # Create 1 device
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=800)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _discard()

    return True


def _clean_up():
    #    linux.umount(lv_mnt)

    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    try:
        start_test()
    except Exception as e:
        print(e)
        traceback.print_exc()
        e = sys.exc_info()[0]
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def access_group_create_success():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Creating AG %s with protocol %s." % (ag_name, config['protocol']),
                               command=lsm.access_group_create,
                               name=ag_name,
                               init=init_id,
                               sys=sys_id,
                               return_output=True,
                               errors=errors)
        ag_id = [line.split()[0].strip() for line in data.splitlines() if ag_name in line][0]
        atomic_run("Deleting AG %s with protocol %s." % (ag_name, config['protocol']),
                   command=lsm.access_group_delete,
                   ag=ag_id,
                   force=True,
                   errors=errors)
    return errors


def access_group_create_fail():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail creating AG without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.access_group_create},
            {'message': "Trying to fail creating AG without name with protocol %s" % config['protocol'],
             'command': lsm.access_group_create, 'sys': sys_id, 'init': init_id},
            {'message': "Trying to fail creating AG without init with protocol %s" % config['protocol'],
             'command': lsm.access_group_create, 'sys': sys_id, 'name': ag_name},
            {'message': "Trying to fail creating AG without sys with protocol %s" % config['protocol'],
             'command': lsm.access_group_create, 'init': init_id, 'name': ag_name},
            {'message': "Trying to fail creating AG with wrong init '-1' with protocol %s" % config['protocol'],
             'command': lsm.access_group_create, 'init': -1, 'name': ag_name, 'sys': sys_id},
            {'message': "Trying to fail creating AG with NONEXISTENT sys with protocol %s" % config['protocol'],
             'command': lsm.access_group_create, 'init': init_id, 'name': ag_name, 'sys': "NONEXISTENT"},
            {'message': "Trying to fail creating AG with NONEXISTENT init with protocol %s" % config['protocol'],
             'command': lsm.access_group_create, 'init': "NONEXISTENT", 'name': ag_name, 'sys': sys_id}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def access_group_create_fail_name_conflict():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Creating AG %s to hit NAME CONFLICT with protocol %s." % (ag_name, config['protocol']),
                   command=lsm.access_group_create,
                   name=ag_name,
                   init=init_id,
                   sys=sys_id,
                   expected_ret=4,
                   errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = access_group_create_success()
    if int(environ['fmf_tier']) == 2:
        errs = access_group_create_fail()
        errs += access_group_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_clone_success():
    errors = []

    clone_name = read_var("LSM_FS_CLONED_NAME")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Cloning FS %s to FS %s with protocol %s." % (fs_id, clone_name, config['protocol']),
                               command=lsm.fs_clone,
                               src_fs=fs_id,
                               dst_name=clone_name,
                               return_output=True,
                               errors=errors)
        cloned_fs_id = [line.split()[0].strip() for line in data.splitlines() if clone_name in line][0]
        atomic_run("Removing cloned FS %s with protocol %s." % (clone_name, config['protocol']),
                   command=lsm.fs_delete,
                   fs=cloned_fs_id,
                   force=True,
                   errors=errors)
        # FIXME: Add test with backing-snapshot
    return errors


def fs_clone_fail():
    errors = []

    clone_name = read_var("LSM_FS_CLONED_NAME")
    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail cloning FS without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.fs_clone},
            {'message': "Trying to fail cloning FS without 'dst-name' with protocol %s" % config['protocol'],
             'command': lsm.fs_clone, 'src_fs': fs_id},
            {'message': "Trying to fail cloning FS without 'src-fs' with protocol %s" % config['protocol'],
             'command': lsm.fs_clone, 'dst_name': clone_name},
            {'message': "Trying to fail cloning FS with wrong 'src-fs' 'wrong' with protocol %s" % config['protocol'],
             'command': lsm.fs_clone, 'dst_name': clone_name, 'src_fs': "wrong"},
            {'message': "Trying to fail cloning FS by giving 'src-fs' name instead of ID with protocol %s" % config['protocol'],
             'command': lsm.fs_clone, 'dst_name': clone_name, 'src_fs': fs_name}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def fs_clone_fail_name_conflict():
    errors = []

    clone_name = read_var("LSM_FS_CLONED_NAME")
    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Cloning FS %s to FS %s with protocol %s." % (fs_id, clone_name, config['protocol']),
                               command=lsm.fs_clone,
                               src_fs=fs_id,
                               dst_name=clone_name,
                               return_output=True,
                               errors=errors)
        cloned_fs_id = [line.split()[0].strip() for line in data.splitlines() if clone_name in line][0]

        atomic_run("Cloning FS %s again to hit NAME CONFLICT with cloned FS name with protocol %s." % (fs_name, config['protocol']),
                   command=lsm.fs_clone,
                   src_fs=fs_id,
                   dst_name=clone_name,
                   return_output=True,
                   expected_ret=4,
                   errors=errors)

        atomic_run("Cloning FS %s again to hit NAME CONFLICT with origin FS name with protocol %s." % (fs_name, config['protocol']),
                   command=lsm.fs_clone,
                   src_fs=fs_id,
                   dst_name=fs_name,
                   return_output=True,
                   expected_ret=4,
                   errors=errors)

        atomic_run("Removing cloned FS %s with protocol %s." % (clone_name, config['protocol']),
                   command=lsm.fs_delete,
                   fs=cloned_fs_id,
                   force=True,
                   errors=errors)
    return errors

if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_clone_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_clone_fail()
        errs += fs_clone_fail_name_conflict()
    exit(parse_ret(errs))
